package io.xjar.jar;

import java.io.Closeable;
import io.xjar.XKit;
import java.util.jar.Attributes;
import java.util.jar.Manifest;
import org.apache.commons.compress.archivers.ArchiveEntry;
import io.xjar.XUnclosedOutputStream;
import io.xjar.XUnclosedInputStream;
import org.apache.commons.compress.archivers.jar.JarArchiveOutputStream;
import org.apache.commons.compress.archivers.jar.JarArchiveInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.File;
import io.xjar.key.XKey;
import io.xjar.XEntryFilter;
import io.xjar.XConstants;
import io.xjar.XDecryptor;
import org.apache.commons.compress.archivers.jar.JarArchiveEntry;
import io.xjar.XEntryDecryptor;

public class XJarDecryptor extends XEntryDecryptor<JarArchiveEntry> implements XDecryptor, XConstants
{
    private final int level;
    
    public XJarDecryptor(final XDecryptor xDecryptor) {
        this(xDecryptor, (XEntryFilter<JarArchiveEntry>)new XJarAllEntryFilter());
    }
    
    public XJarDecryptor(final XDecryptor xDecryptor, final XEntryFilter<JarArchiveEntry> filter) {
        this(xDecryptor, -1, filter);
    }
    
    public XJarDecryptor(final XDecryptor xDecryptor, final int level) {
        this(xDecryptor, level, (XEntryFilter<JarArchiveEntry>)new XJarAllEntryFilter());
    }
    
    public XJarDecryptor(final XDecryptor xDecryptor, final int level, final XEntryFilter<JarArchiveEntry> filter) {
        super(xDecryptor, (XEntryFilter)filter);
        this.level = level;
    }
    
    public void decrypt(final XKey key, final File src, final File dest) throws IOException {
        try (FileInputStream fis = new FileInputStream(src);
             FileOutputStream fos = new FileOutputStream(dest)) {
            this.decrypt(key, fis, fos);
        }
    }
    
    public void decrypt(final XKey key, final InputStream in, final OutputStream out) throws IOException {
        JarArchiveInputStream zis = null;
        JarArchiveOutputStream zos = null;
        try {
            zis = new JarArchiveInputStream(in);
            zos = new JarArchiveOutputStream(out);
            zos.setLevel(this.level);
            final XUnclosedInputStream nis = new XUnclosedInputStream((InputStream)zis);
            final XUnclosedOutputStream nos = new XUnclosedOutputStream((OutputStream)zos);
            JarArchiveEntry entry;
            while ((entry = zis.getNextJarEntry()) != null) {
                if (!entry.getName().startsWith(XJarDecryptor.XJAR_SRC_DIR) && !entry.getName().endsWith("XJAR-INF/")) {
                    if (entry.getName().endsWith("XJAR-INF/INDEXES.IDX")) {
                        continue;
                    }
                    if (entry.isDirectory()) {
                        final JarArchiveEntry jarArchiveEntry = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry);
                    }
                    else if (entry.getName().equals("META-INF/MANIFEST.MF")) {
                        final Manifest manifest = new Manifest((InputStream)nis);
                        final Attributes attributes = manifest.getMainAttributes();
                        final String mainClass = attributes.getValue("Jar-Main-Class");
                        if (mainClass != null) {
                            attributes.putValue("Main-Class", mainClass);
                            attributes.remove(new Attributes.Name("Jar-Main-Class"));
                        }
                        final JarArchiveEntry jarArchiveEntry2 = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry2.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry2);
                        manifest.write((OutputStream)nos);
                    }
                    else {
                        final JarArchiveEntry jarArchiveEntry = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry);
                        final boolean filtered = this.filtrate((Object)entry);
                        final XDecryptor decryptor = (XDecryptor)(filtered ? this.xDecryptor : this.xNopDecryptor);
                        try (OutputStream eos = decryptor.decrypt(key, (OutputStream)nos)) {
                            XKit.transfer((InputStream)nis, eos);
                        }
                    }
                    zos.closeArchiveEntry();
                }
            }
            zos.finish();
        }
        finally {
            XKit.close((Closeable)zis);
            XKit.close((Closeable)zos);
        }
    }
    
    public boolean predicate(final InputStream jar) throws IOException {
        JarArchiveInputStream zis = null;
        try {
            zis = new JarArchiveInputStream(jar);
            JarArchiveEntry entry;
            while ((entry = zis.getNextJarEntry()) != null) {
                if (!entry.getName().startsWith(XJarDecryptor.XJAR_SRC_DIR) && !entry.getName().endsWith("XJAR-INF/") && !entry.getName().endsWith("XJAR-INF/INDEXES.IDX") && !entry.isDirectory()) {
                    if (entry.getName().equals("META-INF/MANIFEST.MF")) {
                        continue;
                    }
                    if (this.filtrate((Object)entry)) {
                        return true;
                    }
                    continue;
                }
            }
            return false;
        }
        finally {
            XKit.close((Closeable)zis);
        }
    }
}
